@@ -1,7 +1,7 @@
 # ============================================================
 #
 # Copyright (C) 2010 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
-# Copyright (C) 2011, 2012, 2013, 2014, 2015 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
+# Copyright (C) 2011-2016 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 #
 # This file may be licensed under the terms of the
 # GNU Lesser General Public License Version 3 (the ``LGPL''),
@@ -194,6 +194,21 @@ class QualityOfServiceSpec(object):
                                self.__reliability)
 
 
+CONFIG_DEBUG_VARIABLE = 'RSB_CONFIG_DEBUG'
+
+CONFIG_FILES_VARIABLE = 'RSB_CONFIG_FILES'
+
+CONFIG_FILE_KEY_SYSTEM = '%system'
+CONFIG_FILE_KEY_PREFIX = '%prefix'
+CONFIG_FILE_KEY_USER = '%user'
+CONFIG_FILE_KEY_PWD = '%pwd'
+
+DEFAULT_CONFIG_FILES = [CONFIG_FILE_KEY_SYSTEM,
+                        CONFIG_FILE_KEY_PREFIX,
+                        CONFIG_FILE_KEY_USER,
+                        CONFIG_FILE_KEY_PWD]
+
+
 def _configFileToDict(path, defaults=None):
     parser = ConfigParser.RawConfigParser()
     parser.read(path)
@@ -207,25 +222,40 @@ def _configFileToDict(path, defaults=None):
     return options
 
 
-def _configEnvironmentToDict(defaults=None):
+def _configEnvironmentToDict(defaults=None, debug=False):
     if defaults is None:
         options = {}
     else:
         options = defaults
+    empty = True
     for (key, value) in os.environ.items():
         if key.startswith('RSB_'):
-            if value == '':
+            if debug:
+                empty = False
+                print('     %s -> %s' % (key, value))
+            if not key == CONFIG_FILES_VARIABLE and value == '':
                 raise ValueError('The value of the environment variable '
                                  '%s is the empty string' % key)
             options[key[4:].lower().replace('_', '.')] = value
+    if debug and empty:
+        print('     <none>')
     return options
 
 
-def _configDefaultSourcesToDict(defaults=None):
+def _configDefaultConfigFiles():
+    if CONFIG_FILES_VARIABLE in os.environ:
+        return [f for f in os.environ[CONFIG_FILES_VARIABLE].split(':') if f]
+    else:
+        return DEFAULT_CONFIG_FILES
+
+
+def _configDefaultSourcesToDict(defaults=None,
+                                files=_configDefaultConfigFiles()):
     r"""
     Obtain configuration options from multiple sources, store them
-    in a :obj:`ParticipantConfig` object and return it. The following
-    sources of configuration information will be consulted:
+    in a :obj:`ParticipantConfig` object and return it. By default,
+    the following sources of configuration information will be
+    consulted:
 
      1. ``/etc/rsb.conf``
      2. ``$prefix/etc/rsb.conf``
@@ -236,6 +266,11 @@ def _configDefaultSourcesToDict(defaults=None):
     Args:
         defaults (dict of str -> str):
             dictionary with default options
+        files (list of str)
+            filenames and placeholders for configuration files
+
+            The placeholders ``%system``, ``%prefix``, ``%user`` and
+            ``%pwd`` can be used to refer to the sources 1-4 above.
 
     Returns:
         dict of str -> str:
@@ -246,6 +281,7 @@ def _configDefaultSourcesToDict(defaults=None):
         :obj:`_configFileToDict`, :obj:`_configEnvironmentToDict`:
     """
 
+    # Prepare defaults.
     if defaults is None:
         defaults = {}
     if 'transport.socket.enabled' not in defaults:
@@ -256,12 +292,50 @@ def _configDefaultSourcesToDict(defaults=None):
         systemConfigFile = "c:\\rsb.conf"
     else:
         systemConfigFile = "/etc/rsb.conf"
-    partial = _configFileToDict(systemConfigFile, defaults)
-    partial = _configFileToDict("%s/etc/rsb.conf" % rsb.util.prefix(), partial)
-    partial = _configFileToDict(os.path.expanduser("~/.config/rsb.conf"),
-                                partial)
-    partial = _configFileToDict("rsb.conf", partial)
-    return _configEnvironmentToDict(partial)
+
+    # Configure sources.
+    debug = CONFIG_DEBUG_VARIABLE in os.environ
+
+    fileIndex = [1]
+
+    def fromFile(configFile, description):
+        def processFile(partial):
+            if debug:
+                if fileIndex[0] == 1:
+                    print('  1. Configuration files')
+                print('     %d. %s "%s" %s'
+                      % (fileIndex[0], description, configFile,
+                         'exists' if os.path.exists(configFile)
+                         else 'does not exist'))
+                fileIndex[0] += 1
+            return _configFileToDict(configFile, partial)
+        return processFile
+
+    def processEnvironment(partial):
+        if debug:
+            print('  2. Environment variables with prefix RSB_')
+        return _configEnvironmentToDict(partial, debug=debug)
+
+    def processSpec(spec):
+        if spec == CONFIG_FILE_KEY_SYSTEM:
+            return fromFile(systemConfigFile,
+                            'System wide config file')
+        elif spec == CONFIG_FILE_KEY_PREFIX:
+            return fromFile('%s/etc/rsb.conf' % rsb.util.prefix(),
+                            'Prefix wide config file')
+        elif spec == CONFIG_FILE_KEY_USER:
+            return fromFile(os.path.expanduser('~/.config/rsb.conf'),
+                            'User config file')
+        elif spec == CONFIG_FILE_KEY_PWD:
+            return fromFile('rsb.conf', 'Current directory file')
+        else:
+            return fromFile(spec, 'User specified config file')
+    sources = [processSpec(f) for f in files] + [processEnvironment]
+
+    # Merge sources and defaults.
+    if debug:
+        print('Configuring with sources (lowest priority first)')
+    return reduce(lambda partial, source: source(partial), sources, defaults)
 
 _CONFIG_TRUE_VALUES = ['1', 'true', 'yes']
 
@@ -960,11 +1034,11 @@ class MetaData(object):
         return not self.__eq__(other)
 
     def __str__(self):
-        return '%s[create = %s, send = %s, receive = %s, deliver = %s, ' \
-            'userTimes = %s, userInfos = %s]' \
-            % ('MetaData',
-               self.__createTime, self.__sendTime, self.__receiveTime,
-               self.__deliverTime, self.__userTimes, self.__userInfos)
+        return ('%s[createTime= %s, sendTime = %s, receiveTime = %s, '
+                'deliverTime = %s, userTimes = %s, userInfos = %s]'
+                % ('MetaData',
+                   self.__createTime, self.__sendTime, self.__receiveTime,
+                   self.__deliverTime, self.__userTimes, self.__userInfos))
 
     def __repr__(self):
         return self.__str__()
