@@ -1,7 +1,7 @@
 # ============================================================
 #
 # Copyright (C) 2011 by Johannes Wienke <jwienke at techfak dot uni-bielefeld dot de>
-# Copyright (C) 2011, 2012 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
+# Copyright (C) 2011, 2012, 2015 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 #
 # This file may be licensed under the terms of the
 # GNU Lesser General Public License Version 3 (the ``LGPL''),
@@ -24,11 +24,11 @@
 # ============================================================
 
 """
-A module with classes maintaining the processing of events between the transport
-layer and the client interface.
+A module with classes maintaining the processing of events between the
+transport layer and the client interface.
 
-@author: jwienke
-@author: jmoringe
+.. codeauthor:: jwienke
+.. codeauthor:: jmoringe
 """
 
 import copy
@@ -38,12 +38,13 @@ import Queue
 import rsb.util
 import rsb.filter
 
-class BroadcastProcessor (object):
+
+class BroadcastProcessor(object):
     """
     This event processor implements synchronous broadcast dispatch to
     a list of handlers.
 
-    @author: jmoringe
+    .. codeauthor:: jmoringe
     """
     def __init__(self, handlers=None):
         self.__logger = rsb.util.getLoggerByClass(self.__class__)
@@ -80,43 +81,43 @@ class BroadcastProcessor (object):
                len(self.handlers),
                id(self))
 
+
 class EventReceivingStrategy(object):
     """
     Superclass for event receiving strategies.
 
-    @author: jmoringe
+    .. codeauthor:: jmoringe
     """
-    def __init__(self):
-        pass
-
     def addFilter(self, theFilter):
         raise NotImplementedError
 
     def removeFilter(self, theFilter):
         raise NotImplementedError
 
-    def addHandler(self, handler):
+    def addHandler(self, handler, wait):
         raise NotImplementedError
 
-    def removeHandler(self, hanlder):
+    def removeHandler(self, handler, wait):
         raise NotImplementedError
 
-    def handle(self):
+    def handle(self, event):
         raise NotImplementedError
 
 
 class ParallelEventReceivingStrategy(EventReceivingStrategy):
     """
-    An L{EventReceivingStrategy} that dispatches events to multiple
+    An :obj:`EventReceivingStrategy` that dispatches events to multiple
     handlers in individual threads in parallel. Each handler is called only
     sequentially but potentially from different threads.
 
-    @author: jwienke
+    .. codeauthor:: jwienke
     """
 
     def __init__(self, numThreads=5):
         self.__logger = rsb.util.getLoggerByClass(self.__class__)
-        self.__pool = rsb.util.OrderedQueueDispatcherPool(threadPoolSize=numThreads, delFunc=self.__deliver, filterFunc=self.__filter)
+        self.__pool = rsb.util.OrderedQueueDispatcherPool(
+            threadPoolSize=numThreads, delFunc=self.__deliver,
+            filterFunc=self.__filter)
         self.__pool.start()
         self.__filters = []
         self.__filtersMutex = threading.RLock()
@@ -132,6 +133,7 @@ class ParallelEventReceivingStrategy(EventReceivingStrategy):
             self.__pool = None
 
     def __deliver(self, action, event):
+        # pylint: disable=no-self-use
         action(event)
 
     def __filter(self, action, event):
@@ -147,8 +149,9 @@ class ParallelEventReceivingStrategy(EventReceivingStrategy):
         """
         Dispatches the event to all registered listeners.
 
-        @type event: Event
-        @param event: event to dispatch
+        Args:
+            event:
+                event to dispatch
         """
         self.__logger.debug("Processing event %s", event)
         event.metaData.setDeliverTime()
@@ -168,13 +171,14 @@ class ParallelEventReceivingStrategy(EventReceivingStrategy):
         with self.__filtersMutex:
             self.__filters.append(theFilter)
 
+
 class FullyParallelEventReceivingStrategy(EventReceivingStrategy):
     """
-    An L{EventReceivingStrategy} that dispatches events to multiple
+    An :obj:`EventReceivingStrategy` that dispatches events to multiple
     handlers in individual threads in parallel. Each handler can be called
     in parallel for different requests.
 
-    @author: jwienke
+    .. codeauthor:: jwienke
     """
 
     def __init__(self):
@@ -206,8 +210,9 @@ class FullyParallelEventReceivingStrategy(EventReceivingStrategy):
         """
         Dispatches the event to all registered listeners.
 
-        @type event: Event
-        @param event: event to dispatch
+        Args:
+            event:
+                event to dispatch
         """
         self.__logger.debug("Processing event %s", event)
         event.metaData.setDeliverTime()
@@ -233,16 +238,17 @@ class FullyParallelEventReceivingStrategy(EventReceivingStrategy):
         with self.__mutex:
             self.__filters.append(f)
 
+
 class NonQueuingParallelEventReceivingStrategy(EventReceivingStrategy):
     """
-    An L{EventReceivingStrategy} that dispatches events to multiple
+    An :obj:`EventReceivingStrategy` that dispatches events to multiple
     handlers using a single thread and without queuing. Only a single buffer
     is used to decouple the transport from the registered handlers. In case
     the handler processing is slower than the transport, the transport will
     block on inserting events into this strategy. Callers must ensure that they
     are in no active call for #handle when deactivating this instance.
 
-    @author: jwienke
+    .. codeauthor:: jwienke
     """
 
     def __init__(self):
@@ -297,7 +303,7 @@ class NonQueuingParallelEventReceivingStrategy(EventReceivingStrategy):
             self.__filters.append(f)
 
 
-class EventSendingStrategy (object):
+class EventSendingStrategy(object):
     def getConnectors(self):
         raise NotImplementedError
 
@@ -312,7 +318,8 @@ class EventSendingStrategy (object):
     def handle(self, event):
         raise NotImplementedError
 
-class DirectEventSendingStrategy (EventSendingStrategy):
+
+class DirectEventSendingStrategy(EventSendingStrategy):
     def __init__(self):
         self.__connectors = []
 
@@ -329,14 +336,15 @@ class DirectEventSendingStrategy (EventSendingStrategy):
         for connector in self.__connectors:
             connector.handle(event)
 
-class Configurator (object):
+
+class Configurator(object):
     """
     Superclass for in- and out-direction Configurator classes. Manages
     the basic aspects like the connector list and (de)activation that
     are not direction-specific.
 
-    @author: jwienke
-    @author: jmoringe
+    .. codeauthor:: jwienke
+    .. codeauthor:: jmoringe
     """
     def __init__(self, connectors=None):
         self.__logger = rsb.util.getLoggerByClass(self.__class__)
@@ -361,8 +369,9 @@ class Configurator (object):
         Defines the scope the in route has to be set up. This will be called
         before calling #activate.
 
-        @param scope: the scope of the in route
-        @type scope: rsb.Scope
+        Args:
+            scope (rsb.Scope):
+                the scope of the in route
         """
         self.__scope = scope
         self.__logger.debug("Got new scope %s", scope)
@@ -376,6 +385,19 @@ class Configurator (object):
 
     connectors = property(getConnectors)
 
+    def getTransportURLs(self):
+        """
+        Return list of transport URLs describing the connectors
+        managed by the configurator.
+
+        Returns:
+            list:
+                List of transport URLs.
+        """
+        return set(x.getTransportURL() for x in self.__connectors)
+
+    transportURLs = property(getTransportURLs)
+
     def isActive(self):
         return self.__active
 
@@ -383,7 +405,7 @@ class Configurator (object):
 
     def activate(self):
         if self.__active:
-            raise RuntimeError, "Configurator is already active"
+            raise RuntimeError("Configurator is already active")
 
         self.__logger.info("Activating configurator")
         for connector in self.connectors:
@@ -393,7 +415,7 @@ class Configurator (object):
 
     def deactivate(self):
         if not self.__active:
-            raise RuntimeError, "Configurator is not active"
+            raise RuntimeError("Configurator is not active")
 
         self.__logger.info("Deactivating configurator")
         for connector in self.connectors:
@@ -405,27 +427,28 @@ class Configurator (object):
         for connector in self.connectors:
             connector.setQualityOfServiceSpec(qos)
 
+
 class InRouteConfigurator(Configurator):
     """
     Instances of this class manage the receiving, filtering and
-    dispatching of events via one or more L{rsb.transport.Connector} s
-    and an L{EventReceivingStrategy}.
+    dispatching of events via one or more :obj:`rsb.transport.Connector` s
+    and an :obj:`EventReceivingStrategy`.
 
-    @author: jwienke
-    @author: jmoringe
+    .. codeauthor:: jwienke
+    .. codeauthor:: jmoringe
     """
 
     def __init__(self, connectors=None, receivingStrategy=None):
         """
-        Creates a new configurator which manages B{connectors} and
-        B{receivingStrategy}.
+        Creates a new configurator which manages ``connectors`` and
+        ``receivingStrategy``.
 
-        @param connectors: Connectors through which events are received.
-
-        @param receivingStrategy: The event receiving strategy
-                                  according to which the filtering and
-                                  dispatching of incoming events
-                                  should be performed.
+        Args:
+            connectors:
+                Connectors through which events are received.
+            receivingStrategy:
+                The event receiving strategy according to which the filtering
+                and dispatching of incoming events should be performed.
         """
         super(InRouteConfigurator, self).__init__(connectors)
 
@@ -457,12 +480,13 @@ class InRouteConfigurator(Configurator):
         for connector in self.connectors:
             connector.filterNotify(theFilter, rsb.filter.FilterAction.ADD)
 
+
 class OutRouteConfigurator(Configurator):
     """
     Instances of this class manage the sending of events via one or
-    more L{rsb.transport.Connector} s and an L{EventSendingStrategy}.
+    more :obj:`rsb.transport.Connector` s and an :obj:`EventSendingStrategy`.
 
-    @author: jmoringe
+    .. codeauthor:: jmoringe
     """
 
     def __init__(self, connectors=None, sendingStrategy=None):
@@ -475,12 +499,13 @@ class OutRouteConfigurator(Configurator):
         else:
             self.__sendingStrategy = sendingStrategy
 
-        if not connectors is None:
+        if connectors is not None:
             map(self.__sendingStrategy.addConnector, connectors)
 
     def handle(self, event):
         if not self.active:
-            raise RuntimeError, "Trying to publish event on Configurator which is not active."
+            raise RuntimeError("Trying to publish event on Configurator "
+                               "which is not active.")
 
         self.__logger.debug("Publishing event: %s", event)
         self.__sendingStrategy.handle(event)
